
/*******************************************************************************
Figure 15.7 A simple forward chaining interpreter for if-then rules.

Data for backward and forward chained interpreters is the same.

Query: forward.

Problem: (1) Need to clean up fact and already_asked predicates.
Solution: (1) Have a clean up clause.
*******************************************************************************/

:- dynamic(fact/1).

:-  op(800, fx, if).
:-  op(700, xfx, then).
:-  op(300, xfy, or).
:-  op(200, xfy, and).

forward  :-
    new_derived_fact(P),            % A new fact
    !,
    write('Derived: '), write(P), nl,
    assert(fact(P)),
    forward                         % Continue
    ;
    write('No more facts').         % All facts derived

new_derived_fact(Concl) :-
    if Cond then Concl,             % A rule
    \+ fact(Concl),                 % Rule's conclusion not yet a fact
    composed_fact(Cond).            % Condition true?

composed_fact(Cond) :-
    fact(Cond).                     % Simple fact

composed_fact(Cond1 and Cond2) :-
    composed_fact(Cond1),           % Both conjuncts true
    composed_fact(Cond2).

composed_fact(Cond1 or Cond2) :-
    composed_fact(Cond1)            % One disjunct true
    ;
    composed_fact(Cond2).
